/*******************************************************************************
 *                                                                             *
 * File: my3126.c                                                              *
 * Revision: 1.2                                                               *
 * Date: 2005/03/10 07:07:38                                                   *
 * Description:                                                                *
 *  CX-4 PHY functionality.                                                    *
 *  part of the Chelsio 10Gb Ethernet Driver.                                  *
 *                                                                             *
 * This program is free software; you can redistribute it and/or modify        *
 * it under the terms of the GNU General Public License, version 2, as         *
 * published by the Free Software Foundation.                                  *
 *                                                                             *
 * You should have received a copy of the GNU General Public License along     *
 * with this program; if not, write to the Free Software Foundation, Inc.,     *
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.                   *
 *                                                                             *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED      *
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF        *
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.                       *
 *                                                                             *
 * http://www.chelsio.com                                                      *
 *                                                                             *
 * Copyright (c) 2003 - 2005 Chelsio Communications, Inc.                      *
 * All rights reserved.                                                        *
 *                                                                             *
 * Maintainers: maintainers@chelsio.com                                        *
 *                                                                             *
 * History:                                                                    *
 *                                                                             *
 ******************************************************************************/

#include "cphy.h"
#include "elmer0.h"

/* Port Reset */
static int my3126_reset(struct cphy *cphy, int wait)
{
	/* This can be done through registers.  It is not required since
	 * a full chip reset is used.
	 */
	return 0;
}

static int my3126_interrupt_enable(struct cphy *cphy)
{
	return 0;
}

static int my3126_interrupt_disable(struct cphy *cphy)
{
	return 0;
}

static int my3126_interrupt_clear(struct cphy *cphy)
{
	return 0;
}

static int my3126_interrupt_handler(struct cphy *cphy)
{
	my3126_interrupt_clear(cphy);

	/* We have only enabled link change interrupts so it must be that */
	return cphy_cause_link_change;
}

static int my3126_set_loopback(struct cphy *cphy, int on)
{
	return 0;
}

static int my3126_get_link_status(struct cphy *cphy, int *link_ok, int *speed,
				  int *duplex, int *fc)
{
	return 0;
}

static void my3126_destroy(struct cphy *cphy)
{
	t1_free((void *) cphy, sizeof(*cphy));
}

static struct cphy_ops my3126_ops = {
	.destroy           = my3126_destroy,
	.reset             = my3126_reset,
	.interrupt_enable  = my3126_interrupt_enable,
	.interrupt_disable = my3126_interrupt_disable,
	.interrupt_clear   = my3126_interrupt_clear,
	.interrupt_handler = my3126_interrupt_handler,
	.get_link_status   = my3126_get_link_status,
	.set_loopback      = my3126_set_loopback,
};

static struct cphy *my3126_phy_create(adapter_t *adapter, int phy_addr,
				      struct mdio_ops *mdio_ops)
{
	struct cphy *cphy = t1_malloc(sizeof(*cphy));

	if (cphy)
		cphy_init(cphy, adapter, phy_addr, &my3126_ops, mdio_ops);
	return cphy;
}

/* Chip Reset */
static int my3126_phy_reset(adapter_t * adapter)
{
	u32 val;

	t1_tpi_read(adapter, A_ELMER0_GPO, &val);
	val &= ~4;
	t1_tpi_write(adapter, A_ELMER0_GPO, val);
	DELAY_MS(100);

	t1_tpi_write(adapter, A_ELMER0_GPO, val | 4);
	DELAY_MS(1000);

	/* Now lets enable the Laser. Delay 100us */
	t1_tpi_read(adapter, A_ELMER0_GPO, &val);
	val |= 0x8000;
	t1_tpi_write(adapter, A_ELMER0_GPO, val);
	DELAY_US(100);
	return 0;
}

struct gphy t1_my3126_ops = {
	my3126_phy_create,
	my3126_phy_reset
};
