#include <linux/version.h>

/********************************************************************/
/* General backward compatibility                                   */
/********************************************************************/

#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 19)
#define BUG_ON(condition) do { if ((condition)!=0) BUG(); } while(0)
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
#define WARN_ON(condition) BUG_ON(condition)
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
#define in_atomic in_interrupt
#endif

#ifndef __devexit_p
#define __devexit_p(x) x
#endif

#ifndef ALIGN
#define ALIGN(x,a) (((x)+(a)-1)&~((a)-1))
#endif

/* Work queue / task queue compatibility */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,41)
#include <linux/tqueue.h>
#define work_struct tq_struct
#define INIT_WORK(x,y,z) INIT_TQUEUE(x,y,z)
#define schedule_work(x) schedule_task(x)
#endif

#ifndef module_param
#define module_param(name,type,perm) MODULE_PARM(name,"i")
#endif

/* Interrupt handler backwards compatibility */
#ifndef IRQ_NONE
#define IRQ_NONE
#define IRQ_HANDLED
typedef void irqreturn_t;
#endif

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,4,20))
static inline void list_move(struct list_head *list, struct list_head *head)
{
        __list_del(list->prev, list->next);
        list_add(list, head);
}

static inline void list_move_tail(struct list_head *list,
				  struct list_head *head)
{
        __list_del(list->prev, list->next);
        list_add_tail(list, head);
}
#endif

#ifndef HAVE_NETDEV_PRIV
#define netdev_priv(dev)	((dev)->priv)
#endif /* ! HAVE_NETDEV_PRIV */

/********************************************************************/
/* Network backward compatibility                                   */
/********************************************************************/

/* Device type while in monitor mode, appeared in Linux 2.4.6 */
#ifndef ARPHRD_IEEE80211
#define ARPHRD_IEEE80211 801
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
#ifndef HAVE_FREE_NETDEV
#define free_netdev(dev) kfree(dev)
#endif
#endif

#ifndef SET_NETDEV_DEV
#define SET_NETDEV_DEV(x,y)
#endif


/********************************************************************/
/* Wireless extensions backward compatibility                       */
/********************************************************************/

#if defined(WIRELESS_EXT) && (WIRELESS_EXT <= 12)
/* Taken verbatim from iw_handler.h */
struct iw_request_info
{
	u16 cmd;	/* Wireless Extension command */
	u16 flags;	/* More to come ;-) */
};

#ifndef SIOCIWFIRSTPRIV
#define SIOCIWFIRSTPRIV	0x8BE0
#endif

#endif	/* WIRELESS_EXT <= 12 */

#ifndef IW_MODE_MONITOR
#define IW_MODE_MONITOR 6
#endif


/********************************************************************/
/* PCI backward compatibility                                       */
/********************************************************************/

#ifdef LINUX_PCI_H
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 22)
static inline char *pci_name(struct pci_dev *pdev)
{
	        return pdev->slot_name;
}
#endif
#endif /* LINUX_PCI_H */


/********************************************************************/
/* PCMCIA backward compatibility                                    */
/********************************************************************/

#ifdef _LINUX_DS_H

/* pcmcia_core module from pcmcia-cs doesn't export pcmcia_* functions
 * and requires using CardServices instead.  Taken from hostap driver. */
#ifdef __IN_PCMCIA_PACKAGE__
#undef pcmcia_access_configuration_register
#define pcmcia_access_configuration_register(handle, reg) \
	CardServices(AccessConfigurationRegister, handle, reg)

#undef pcmcia_register_client
#define pcmcia_register_client(handle, reg) \
	CardServices(RegisterClient, handle, reg)

#undef pcmcia_deregister_client
#define pcmcia_deregister_client(handle) \
	CardServices(DeregisterClient, handle)

#undef pcmcia_get_first_tuple
#define pcmcia_get_first_tuple(handle, tuple) \
	CardServices(GetFirstTuple, handle, tuple)

#undef pcmcia_get_next_tuple
#define pcmcia_get_next_tuple(handle, tuple) \
	CardServices(GetNextTuple, handle, tuple)

#undef pcmcia_get_tuple_data
#define pcmcia_get_tuple_data(handle, tuple) \
	CardServices(GetTupleData, handle, tuple)

#undef pcmcia_parse_tuple
#define pcmcia_parse_tuple(handle, tuple, parse) \
	CardServices(ParseTuple, handle, tuple, parse)

#undef pcmcia_get_configuration_info
#define pcmcia_get_configuration_info(handle, config) \
	CardServices(GetConfigurationInfo, handle, config)

#undef pcmcia_request_io
#define pcmcia_request_io(handle, req) \
	CardServices(RequestIO, handle, req)

#undef pcmcia_request_irq
#define pcmcia_request_irq(handle, req) \
	CardServices(RequestIRQ, handle, req)

#undef pcmcia_request_configuration
#define pcmcia_request_configuration(handle, req) \
	CardServices(RequestConfiguration, handle, req)

#undef pcmcia_release_configuration
#define pcmcia_release_configuration(handle) \
	CardServices(ReleaseConfiguration, handle)

#undef pcmcia_release_io
#define pcmcia_release_io(handle, req) \
	CardServices(ReleaseIO, handle, req)

#undef pcmcia_release_irq
#define pcmcia_release_irq(handle, req) \
	CardServices(ReleaseIRQ, handle, req)

#undef pcmcia_release_window
#define pcmcia_release_window(win) \
	CardServices(ReleaseWindow, win)

#undef pcmcia_get_card_services_info
#define pcmcia_get_card_services_info(info) \
	CardServices(GetCardServicesInfo, info)

#undef pcmcia_reset_card
#define pcmcia_reset_card(handle, err) \
	CardServices(ResetCard, handle, err)

#undef pcmcia_validate_cis
#define pcmcia_validate_cis(handle, reg) \
	CardServices(ValidateCIS, handle, reg)
#endif /* __IN_PCMCIA_PACKAGE__ */


#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,70)

/* Old style pcmcia registration */

struct device_driver {
	char			name[DEV_NAME_LEN];
};

struct pcmcia_driver {
	dev_link_t		*(*attach)(void);
	void			(*detach)(dev_link_t *);
	struct module		*owner;
	struct device_driver	drv;
};

static int __init pcmcia_register_driver(struct pcmcia_driver *driver)
{
	servinfo_t serv;

	CardServices(GetCardServicesInfo, &serv);
	if (serv.Revision != CS_RELEASE_CODE) {
		printk(KERN_NOTICE "orinoco_cs: Card Services release "
		       "does not match!\n");
		return -1;
	}

	register_pccard_driver((dev_info_t *) &driver->drv.name,
			       driver->attach, driver->detach);
	return 0;
}


static void __exit pcmcia_unregister_driver(struct pcmcia_driver *driver)
{
	unregister_pccard_driver((dev_info_t *) &driver->drv.name);
}

static void
cs_error(client_handle_t handle, int func, int ret)
{
	error_info_t err = { func, ret };
	CardServices(ReportError, handle, &err);
}

#endif /* LINUX_VERSION_CODE < KERNEL_VERSION(2,5,70) */
#endif /* _LINUX_DS_H */
