/********************************** */
/* Rule Set Based Access Control    */
/* Author and (c) 1999-2005:        */
/*   Amon Ott <ao@rsbac.org>        */
/* Getname functions for CAP module */
/* Last modified: 28/Jan/2005       */
/********************************** */

#include <rsbac/getname.h>
#include <rsbac/cap_getname.h>
#include <rsbac/helpers.h>
#include <rsbac/error.h>

#ifdef __KERNEL__
#include <linux/sched.h>
#include <linux/string.h>
#include <linux/module.h>
#include <rsbac/rkmem.h>
#include <rsbac/debug.h>
#include <rsbac/aci.h>
#else
#include <string.h>
#endif

static char  cap_list[RSBAC_CAP_MAX+1][17] = {
   "CHOWN",
   "DAC_OVERRIDE",
   "DAC_READ_SEARCH",
   "FOWNER",
   "FSETID",
   "KILL",
   "SETGID",
   "SETUID",
   "SETPCAP",
   "LINUX_IMMUTABLE",
   "NET_BIND_SERVICE",
   "NET_BROADCAST",
   "NET_ADMIN",
   "NET_RAW",
   "IPC_LOCK",
   "IPC_OWNER",
   "SYS_MODULE",
   "SYS_RAWIO",
   "SYS_CHROOT",
   "SYS_PTRACE",
   "SYS_PACCT",
   "SYS_ADMIN",
   "SYS_BOOT",
   "SYS_NICE",
   "SYS_RESOURCE",
   "SYS_TIME",
   "SYS_TTY_CONFIG",
   "MKNOD",
   "LEASE",
   "NONE" };

/*****************************************/

#ifdef __KERNEL__
#ifdef CONFIG_RSBAC_CAP_LOG_MISSING
EXPORT_SYMBOL(rsbac_cap_log_missing_cap);

void rsbac_cap_log_missing_cap(int cap)
  {
    char * tmp;
    union rsbac_target_id_t       i_tid;
    union rsbac_attribute_value_t i_attr_val1;

    if(!rsbac_cap_log_missing)
      return;
#if 0 && LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
    if(cap == CAP_SYS_ADMIN)
      return;
#endif
    
    i_tid.process = current->pid;
    if (rsbac_get_attr(CAP,
                       T_PROCESS,
                       i_tid,
                       A_max_caps_user,
                       &i_attr_val1,
                       FALSE))
      {
        rsbac_ds_get_error("rsbac_cap_log_missing_cap()", A_max_caps_user);
      }
    else
      {
        if(!(i_attr_val1.max_caps_user & (1 << cap)))
          {
            tmp = rsbac_kmalloc(RSBAC_MAXNAMELEN);
            if(tmp)
              {
                get_cap_name(tmp, cap);
#ifdef CONFIG_RSBAC_RMSG
                rsbac_printk(KERN_DEBUG
                             "capable(): pid %u(%.15s), uid %u: missing user max_cap %s!\n",
                             current->pid, current->comm,
                             current->uid,
                             tmp);
#endif
#ifdef CONFIG_RSBAC_RMSG_NOSYSLOG
                if (!rsbac_nosyslog)
#endif
                  printk(KERN_DEBUG
                         "capable(): pid %u(%.15s), uid %u: missing user max_cap %s!\n",
                         current->pid, current->comm,
                         current->uid,
                         tmp);
                  rsbac_kfree(tmp);
              }
          }
      }
    if (rsbac_get_attr(CAP,
                       T_PROCESS,
                       i_tid,
                       A_max_caps_program,
                       &i_attr_val1,
                       FALSE))
      {
        rsbac_ds_get_error("rsbac_cap_log_missing_cap()", A_max_caps_program);
      }
    else
      {
        if(!(i_attr_val1.max_caps_program & (1 << cap)))
          {
            tmp = rsbac_kmalloc(RSBAC_MAXNAMELEN);
            if(tmp)
              {
                get_cap_name(tmp, cap);
#ifdef CONFIG_RSBAC_RMSG
                rsbac_printk(KERN_DEBUG
                             "capable(): pid %u(%.15s), uid %u: missing program max_cap %s!\n",
                             current->pid, current->comm,
                             current->uid,
                             tmp);
#endif
#ifdef CONFIG_RSBAC_RMSG_NOSYSLOG
                if (!rsbac_nosyslog)
#endif
                  printk(KERN_DEBUG
                         "capable(): pid %u(%.15s), uid %u: missing program max_cap %s!\n",
                         current->pid, current->comm,
                         current->uid,
                         tmp);
                  rsbac_kfree(tmp);
              }
          }
      }
  }
#endif
#endif

char * get_cap_name(char * name,
                    u_int value)
  {
    if(!name)
      return(NULL);
    if(value > CAP_NONE)
      strcpy(name, "ERROR!");
    else
      strcpy(name, cap_list[value]);
    return(name);
  }

int get_cap_nr(const char * name)
  {
    int i;
    
    if(!name)
      return(RT_NONE);
    for (i = 0; i < CAP_NONE; i++)
      {
        if (!strcmp(name, cap_list[i]))
          {
            return(i);
          }
      }
    return(CAP_NONE);
  }
